package com.dzeio.openhealth.ui.home

import android.animation.ValueAnimator
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.RectF
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.ui.weight.AddWeightDialog
import com.dzeio.openhealth.utils.DrawUtils
import com.dzeio.openhealth.utils.GraphUtils
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import kotlin.math.min
import kotlin.properties.Delegates

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    companion object {
        const val TAG = "${Application.TAG}/HomeFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    private var intake by Delegates.notNull<Float>()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.addWeight.setOnClickListener {
            AddWeightDialog().show(requireActivity().supportFragmentManager, null)
        }

        binding.fragmentHomeWaterAdd.setOnClickListener { _ ->

            val water = viewModel.water.value

            if (water == null) {

                val w = Water()
                w.value = 200
                viewModel.updateWater(w)
            } else {
                water.value += 200
                viewModel.updateWater(water)
            }
        }

        intake = PreferenceManager.getDefaultSharedPreferences(requireContext())
            .getString("water_intake", "1200")?.toFloat() ?: 1200f

        binding.fragmentHomeWaterTotal.text = "${intake.toInt()}ml"

        binding.fragmentHomeWaterRemove.setOnClickListener { _ ->

            val water = viewModel.water.value

            if (water != null) {

                water.value -= 200
                if (water.value == 0) {
                    viewModel.deleteWater(water)
                } else {
                    viewModel.updateWater(water)
                }
            }
        }

        binding.fragmentHomeWaterRemove.setOnClickListener {
            lifecycleScope.launch {

                val item = viewModel.fetchTodayWater().first()
                Log.d(TAG, "Collected latest $it")
                if (item != null) {
                    item.value -= 200
                    if (item.value == 0) {
                        viewModel.deleteWater(item)
                    } else {
                        viewModel.updateWater(item)
                    }
                }

            }
        }

        binding.listWeight.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())
        }

        binding.gotoWaterHome.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavWaterHome())
        }

        GraphUtils.lineChartSetup(
            binding.weightGraph,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            ), MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnBackground
            )
        )
    }

    private fun updateGraph(list: List<Weight>) {
        if (list.isNotEmpty()) {
            val entries = ArrayList<Entry>()
            for (item in list) {
                entries.add(Entry(item.timestamp.toFloat(), item.weight))
            }

            val dataSet = LineDataSet(entries, "Label")

            binding.weightGraph.apply {

                // Apply new dataset
                data = LineData(dataSet)

                // idk what I did but it works lol
                setVisibleXRange(
                    0f, entries[entries.size - 1].x / 1000f
                )

                // BIS... :(
                // Also it invalidate the view so I don't have to call invalidate
                moveViewToX(entries[entries.size - 1].x - 1600000000f)
            }
        }
    }

    override fun onStart() {
        super.onStart()

        lifecycleScope.launchWhenStarted {
            viewModel.fetchWeights().collectLatest {
                updateGraph(it)
            }
            updateWater(0)
            updateWater(1234)
        }

        viewModel.water.observe(viewLifecycleOwner) {
            Log.d(TAG, "${it?.formatTimestamp()} $it")
            if (it != null) {
                updateWater(it.value)
            } else {
                updateWater(0)
            }
        }

    }

    private fun updateWater(water: Int) {
        val oldValue = binding.fragmentHomeWaterCurrent.text.toString().replace("ml", "").toInt()
        binding.fragmentHomeWaterCurrent.text = "${water}ml"

        var width = 1500
        var height = 750

        if (binding.background.width != 0) {
            width = binding.background.width
            height = binding.background.height
        }

        val graph = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
        Log.d("Test2", "$width $height")

        val canvas = Canvas(graph)
        val rect = RectF(
            10f,
            15f,
            90f,
            85f
        )

//        DrawUtils.drawRect(
//            canvas,
//            RectF(
//                0f,
//                0f,
//                100f,
//                100f
//            ),
//            MaterialColors.getColor(
//                requireView(),
//                com.google.android.material.R.attr.colorOnPrimary
//            ),
//            3f
//        )

        DrawUtils.drawArc(
            canvas,
            100f,
            rect,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnPrimary
            ),
            3f
        )

        Log.d("Test", "${min(oldValue.toFloat(), intake)} ${min(water.toFloat(), intake)}")
        ValueAnimator.ofFloat(min(oldValue.toFloat(), intake), min(water.toFloat(), intake))
            .apply {
                duration = 300
                addUpdateListener {
                    DrawUtils.drawArc(
                        canvas,
                        100 * it.animatedValue as Float / intake,
                        rect,
                        MaterialColors.getColor(
                            requireView(),
                            com.google.android.material.R.attr.colorPrimary
                        ), 6f
                    )
                    canvas.save()
                    binding.background.setImageBitmap(graph)
                }
                start()
            }
    }
}